@@ -7,11 +7,11 @@ dynamic languages, like C and C++.  It is also similar to lint.
 
 Contact Info:
   http://pychecker.sourceforge.net/
-  pychecker@metaslash.com
+  pychecker-list@lists.sourceforge.net
 """
 
 # A version # to check against in the main module (checker.py)
 # this will allow us to check if there are two versions of checker
 # in site-packages and local dir
-MAIN_MODULE_VERSION = 1
+MAIN_MODULE_VERSION = 2
 
