@@ -12,10 +12,12 @@ import string
 import types
 import traceback
 import imp
+import re
 
 from pychecker import OP
 from pychecker import Stack
 from pychecker import function
+from pychecker import python
 
 from pychecker import msgs
 from pychecker import utils
@@ -30,12 +32,23 @@ def _checkSelfArg(method, warnings) :
     """Return a Warning if there is no self parameter or
        the first parameter to a method is not self."""
 
+    if not cfg().methodArgName:
+        return
+
     code = method.function.func_code
     err = None
-    if code.co_argcount < 1 :
+    if method.isStaticMethod():
+        if code.co_argcount > 0 and cfg().methodArgName == code.co_varnames[0]:
+            err = msgs.SELF_IS_ARG % 'staticmethod'
+    elif code.co_argcount < 1: 
         err = msgs.NO_METHOD_ARGS % cfg().methodArgName
-    elif code.co_varnames[0] != cfg().methodArgName :
-        err = msgs.SELF_NOT_FIRST_ARG % cfg().methodArgName
+    else:
+        if method.isClassMethod():
+            if code.co_varnames[0] not in cfg().classmethodArgNames:
+                err = msgs.SELF_NOT_FIRST_ARG % \
+                      (cfg().classmethodArgNames, 'class')
+        elif code.co_varnames[0] != cfg().methodArgName:
+            err = msgs.SELF_NOT_FIRST_ARG % (cfg().methodArgName, '')
 
     if err is not None :
         warnings.append(Warning(code, code, err))
@@ -45,8 +58,8 @@ def _checkNoSelfArg(func, warnings) :
     "Return a Warning if there is a self parameter to a function."
 
     code = func.function.func_code
-    if code.co_argcount > 0 and cfg().methodArgName in code.co_varnames :
-        warnings.append(Warning(code, code, msgs.SELF_IS_ARG))
+    if code.co_argcount > 0 and cfg().methodArgName in code.co_varnames:
+        warnings.append(Warning(code, code, msgs.SELF_IS_ARG % 'function'))
 
 
 def _checkSubclass(c1, c2):
@@ -75,8 +88,12 @@ def _checkReturnWarnings(code) :
 
     # if the last return is implicit, check if there are non None returns
     lastReturn = code.returnValues[-1]
-    if not code.starts_and_ends_with_finally and \
-       cfg().checkImplicitReturns and lastReturn[1].isImplicitNone() :
+
+    # Python 2.4 optimizes the dead implicit return out, so we can't
+    # distinguish implicit and explicit "return None"
+    if utils.pythonVersion() < utils.PYTHON_2_4 and \
+           not code.starts_and_ends_with_finally and \
+           cfg().checkImplicitReturns and lastReturn[1].isImplicitNone():
         for line, retval, dummy in code.returnValues[:-1] :
             if not retval.isNone() :
                 code.addWarning(msgs.IMPLICIT_AND_EXPLICIT_RETURNS,
@@ -153,9 +170,12 @@ def _handleNestedCode(func_code, code, codeSource):
         if nested and func_code.co_name != utils.LAMBDA:
             varnames = func_code.co_varnames + \
                      codeSource.calling_code[-1].function.func_code.co_varnames
+        # save the original return value and restore after checking
+        returnValues = code.returnValues
         code.init(function.create_fake(func_code.co_name, func_code, {},
                                        varnames))
         _checkCode(code, codeSource)
+        code.returnValues = returnValues
 
 def _findUnreachableCode(code) :
     # code after RETURN or RAISE is unreachable unless there's a branch to it
@@ -167,14 +187,17 @@ def _findUnreachableCode(code) :
             unreachable[i] = line
 
     # find the index of the last return
-    lastLine, lastItem, lastIndex = code.returnValues[-1]
+    lastLine = lastItem = lastIndex = None
+    if code.returnValues:
+        lastLine, lastItem, lastIndex = code.returnValues[-1]
     if len(code.returnValues) >= 2 :
         lastIndex = code.returnValues[-2][2]
     if code.raiseValues :
         lastIndex = max(lastIndex, code.raiseValues[-1][2])
 
     # remove last return if it's unreachable AND implicit
-    if unreachable.get(lastIndex) == lastLine and lastItem.isImplicitNone() :
+    if unreachable.get(lastIndex) == lastLine and lastItem and \
+           lastItem.isImplicitNone():
         del code.returnValues[-1]
         del unreachable[lastIndex]
 
@@ -195,19 +218,21 @@ def _checkFunction(module, func, c = None, main = 0, in_class = 0) :
 
     code = CodeChecks.Code()
     code.init(func)
+    if main:
+        for key in func.function.func_globals.keys():
+            code.unusedLocals[key] = -1
     codeSource = CodeChecks.CodeSource(module, func, c, main, in_class, code)
     try :
         _checkCode(code, codeSource)
+        if not in_class :
+            _findUnreachableCode(code)
 
-        # handle lambdas
+        # handle lambdas and nested functions
         codeSource.calling_code.append(func)
         for func_code in code.codeObjects.values() :
             _handleNestedCode(func_code, code, codeSource)
         del codeSource.calling_code[-1]
 
-        if not in_class :
-            _findUnreachableCode(code)
-
     except (SystemExit, KeyboardInterrupt) :
         exc_type, exc_value, exc_tb = sys.exc_info()
         raise exc_type, exc_value
@@ -237,7 +262,7 @@ def _checkFunction(module, func, c = None, main = 0, in_class = 0) :
     #   branches in byte code to setup a loop, so subtract off 2/3's of them
     #    / 2 to approximate real branches
     branches = (len(code.branches.keys()) - (2 * code.loops)) / 2
-    lines = (code.lastLineNum - code.func_code.co_firstlineno)
+    lines = (code.getLineNum() - code.func_code.co_firstlineno)
     returns = len(code.returnValues)
     if not main and not in_class :
         args = code.func_code.co_argcount
@@ -287,7 +312,7 @@ def _baseInitCalled(classInitInfo, base, functionsCalled) :
     if baseInit is None or _get_func_info(baseInit) == classInitInfo :
         return 1
 
-    initName = str(base) + _DOT_INIT
+    initName = utils.safestr(base) + _DOT_INIT
     if functionsCalled.has_key(initName) :
         return 1
 
@@ -325,7 +350,7 @@ def _checkBaseClassInit(moduleFilename, c, func_code, funcInfo) :
         for base in c.classObject.__bases__ :
             if not _baseInitCalled(classInitInfo, base, functionsCalled) :
                 warn = Warning(moduleFilename, func_code,
-                               msgs.BASE_CLASS_NOT_INIT % str(base))
+                               msgs.BASE_CLASS_NOT_INIT % utils.safestr(base))
                 warnings.append(warn)
     return warnings
 
@@ -334,7 +359,7 @@ def _checkOverridenMethods(func, baseClasses, warnings) :
     for baseClass in baseClasses :
         if func.func_name != utils.INIT and \
            not function.same_signature(func, baseClass) :
-            err = msgs.METHOD_SIGNATURE_MISMATCH % (func.func_name, str(baseClass))
+            err = msgs.METHOD_SIGNATURE_MISMATCH % (func.func_name, utils.safestr(baseClass))
             warnings.append(Warning(func.func_code, func.func_code, err))
             break
 
@@ -360,7 +385,7 @@ def getBlackList(moduleList) :
             file, path, flags = imp.find_module(badBoy)
             if file :
                 file.close()
-                blacklist.append(path)
+                blacklist.append(normalize_path(path))
         except ImportError :
             pass
     return blacklist
@@ -378,9 +403,14 @@ def getStandardLibrary() :
         except ImportError :
             return None
 
+def normalize_path(path):
+    return os.path.normpath(os.path.normcase(path))
+
 def removeWarnings(warnings, blacklist, std_lib) :
+    if std_lib is not None:
+        std_lib = normalize_path(std_lib)
     for index in range(len(warnings)-1, -1, -1) :
-        filename = warnings[index].file
+        filename = normalize_path(warnings[index].file)
         if filename in blacklist or (std_lib is not None and
                                      utils.startswith(filename, std_lib)) :
             del warnings[index]
@@ -396,10 +426,21 @@ def _updateSuppressions(suppress, warnings) :
         utils.popConfig()
         raise _SuppressionError
 
+_CLASS_NAME_RE = re.compile("<class '([A-Za-z0-9.]+)'>(\\..+)?")
+
 def getSuppression(name, suppressions, warnings) :
     try :
         utils.pushConfig()
 
+        # cheesy hack to deal with new-style classes.  i don't see a
+        # better way to get the name, '<' is an invalid identifier, so
+        # we can reliably check it and extract name from:
+        # <class 'class-name'>[.identifier[.identifier]...]
+        matches = _CLASS_NAME_RE.match(name)
+        if matches:
+            # pull out the names and make a complete identifier (ignore None)
+            name = string.join(filter(None, matches.groups()), '')
+
         suppress = suppressions[0].get(name, None)
         if suppress is not None :
             _updateSuppressions(suppress, warnings)
@@ -456,10 +497,15 @@ except NameError:
 
 def _findClassWarnings(module, c, class_code,
                        globalRefs, warnings, suppressions) :
-    classSuppress = getSuppression(str(c.classObject), suppressions, warnings)
+    try:
+        className = utils.safestr(c.classObject)
+    except TypeError:
+        # goofy __getattr__
+        return
+    classSuppress = getSuppression(className, suppressions, warnings)
     baseClasses = c.allBaseClasses()
     for base in baseClasses :
-        baseModule = str(base)
+        baseModule = utils.safestr(base)
         if '.' in baseModule :
             # make sure we handle import x.y.z
             packages = string.split(baseModule, '.')
@@ -479,9 +525,31 @@ def _findClassWarnings(module, c, class_code,
         func_code = method.function.func_code
         utils.debug("method:", func_code)
 
-        name = str(c.classObject) + '.' + method.function.func_name
+        try:
+            name = utils.safestr(c.classObject) + '.' + method.function.func_name
+        except AttributeError:
+            # func_name may not exist
+            continue
         methodSuppress = getSuppression(name, suppressions, warnings)
 
+        if cfg().checkSpecialMethods:
+            funcname = method.function.func_name
+            if funcname[:2] == '__' == funcname[-2:] and \
+               funcname != '__init__':
+                err = None
+                argCount = python.SPECIAL_METHODS.get(funcname, -1)
+                if argCount != -1:
+                    # if the args are None, it can be any # of args
+                    if argCount is not None:
+                        minArgs = maxArgs = argCount
+                        err = CodeChecks.getFunctionArgErr(funcname,
+                                     func_code.co_argcount, minArgs, maxArgs)
+                else:
+                    err = msgs.NOT_SPECIAL_METHOD % funcname
+
+                if err is not None:
+                    warnings.append(Warning(filename, func_code, err))
+                
         if cfg().checkOverridenMethods :
             _checkOverridenMethods(method.function, baseClasses, warnings)
 
@@ -508,7 +576,11 @@ def _findClassWarnings(module, c, class_code,
         err = msgs.UNUSED_MEMBERS % (string.join(memberList, ', '), c.name)
         warnings.append(Warning(filename, c.getFirstLine(), err))
 
-    newStyleClass = issubclass(c.classObject, object)
+    try:
+        newStyleClass = issubclass(c.classObject, object)
+    except TypeError:
+        # FIXME: perhaps this should warn b/c it may be a class???
+        newStyleClass = 0
 
     slots = c.statics.get('__slots__')
     if slots is not None and cfg().slots:
@@ -516,9 +588,14 @@ def _findClassWarnings(module, c, class_code,
         if not newStyleClass:
             err = msgs.USING_SLOTS_IN_CLASSIC_CLASS % c.name
             warnings.append(Warning(filename, lineNum, err))
-        elif len(slots.data) == 0 and cfg().emptySlots:
-            err = msgs.EMPTY_SLOTS % c.name
-            warnings.append(Warning(filename, lineNum, err))
+        elif cfg().emptySlots:
+            try:
+                if len(slots.data) == 0:
+                    err = msgs.EMPTY_SLOTS % c.name
+                    warnings.append(Warning(filename, lineNum, err))
+            except AttributeError:
+                # happens when slots is an instance of a class w/o __len__
+                pass
 
     if not newStyleClass and property is not None and cfg().classicProperties:
         for static in c.statics.keys():
@@ -526,6 +603,18 @@ def _findClassWarnings(module, c, class_code,
                 err = msgs.USING_PROPERTIES_IN_CLASSIC_CLASS % (static, c.name)
                 warnings.append(Warning(filename, c.lineNums[static], err))
 
+    coerceMethod = c.methods.get('__coerce__')
+    if newStyleClass and coerceMethod:
+        lineNum = coerceMethod.function.func_code.co_firstlineno
+        err = msgs.USING_COERCE_IN_NEW_CLASS % c.name
+        warnings.append(Warning(filename, lineNum, err))
+
+    gettroMethod = c.methods.get('__getattribute__')
+    if not newStyleClass and gettroMethod:
+        lineNum = gettroMethod.function.func_code.co_firstlineno
+        err = msgs.USING_GETATTRIBUTE_IN_OLD_CLASS % c.name
+        warnings.append(Warning(filename, lineNum, err))
+
     if cfg().noDocClass and c.classObject.__doc__ == None :
         method = c.methods.get(utils.INIT, None)
         if method != None :
@@ -571,8 +660,8 @@ def find(moduleList, initialCfg, suppressions = None) :
         _findFunctionWarnings(module, globalRefs, warnings, suppressions)
 
         for c in module.classes.values() :
-            _findClassWarnings(module, c, classCodes.get(c.name),
-                               globalRefs, warnings, suppressions)
+                _findClassWarnings(module, c, classCodes.get(c.name),
+                                   globalRefs, warnings, suppressions)
 
         if cfg().noDocModule and \
            module.module != None and module.module.__doc__ == None :
@@ -604,3 +693,8 @@ def find(moduleList, initialCfg, suppressions = None) :
     if cfg().ignoreStandardLibrary :
         std_lib = getStandardLibrary()
     return removeWarnings(warnings, getBlackList(cfg().blacklist), std_lib)
+
+if 0:
+    # if you want to test w/psyco, include this
+    import psyco
+    psyco.bind(_checkCode)
