@@ -2,11 +2,13 @@
 
 TESTS="test_input/test[1-9]*.py"
 # comment out to use python from path
-PYTHON="python"
-#PYTHON="$HOME/build/python/clean/python"
-# PYTHON="$HOME/build/python/2.2/python"
-#PYTHON="/usr/bin/python"
-# PYTHON="$PYTHON -tt coverage.py -x"
+#PYTHON="python2"
+#PYTHON="$HOME/build/python/2_3/python"
+#PYTHON="$PYTHON -tt coverage.py -x"
+
+if [ "$PYTHON" = "" ]; then
+    PYTHON=python
+fi
 
 if [ $# -gt 0 ]; then
     TESTS=""
@@ -36,8 +38,14 @@ for test_file in $TESTS ; do
         continue
     fi
 
+    # make sure to use the -F option for this special test
+    extra_args=""
+    if [ "$test_file" = "test_input/test39.py" ]; then
+        extra_args="-F test_input/pycheckrc"
+    fi
+
     test_path=$TMP/$test_name
-    $PYTHON -tt ./pychecker/checker.py --moduledoc --classdoc --no-argsused $test_file > $test_path 2>&1
+    $PYTHON -tt ./pychecker/checker.py --moduledoc --classdoc --no-argsused $extra_args $test_file > $test_path 2>&1
     diff $test_path $expected
     if [ $? -ne 0 ]; then
         error=`expr $error + 1`
