@@ -28,3 +28,25 @@ def g():
     a = `a`
     a = -a
     a = ~a
+
+def h():
+  a = 1
+  a += a
+  a -= a
+  a *= a
+
+  a = not a
+  a = not not a
+
+  a = (1, 2, 3)
+  a, b, c = a
+
+def i(a):
+    z = hash(a.x) & hash(a.y)
+    z = hash(a.x) | hash(a.y)
+    z = hash(a.x) ^ hash(a.y)
+    return z
+
+def j(x, y):
+    return (x + 1.0) / (y + 2.0)
+
